package de.mdelab.mlsdm.interpreter.debug.ui.mlsdmEditor;

import java.io.IOException;
import java.util.Iterator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.model.ITerminate;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.ui.URIEditorInput;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ExtensibleURIConverterImpl;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.workspace.util.WorkspaceSynchronizer;
import org.eclipse.gmf.runtime.diagram.core.services.ViewService;
import org.eclipse.gmf.runtime.diagram.ui.resources.editor.document.IDiagramDocument;
import org.eclipse.gmf.runtime.diagram.ui.resources.editor.document.IDocument;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.part.FileEditorInput;

import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.diagram.edit.parts.ActivityEditPart;
import de.mdelab.mlsdm.diagram.part.Messages;
import de.mdelab.mlsdm.diagram.part.MlsdmDiagramEditorPlugin;
import de.mdelab.mlsdm.diagram.part.MlsdmDocumentProvider;

public class MLSDMDebuggerDocumentProvider extends MlsdmDocumentProvider {
	
	private final ITerminate terminateDebugElement;
	
	public MLSDMDebuggerDocumentProvider( final ITerminate p_terminateDebugElement ) {
		terminateDebugElement = p_terminateDebugElement;
	}
	
	private class MLSDMDebuggerResourceSetInfo extends ResourceSetInfo {
		
		public MLSDMDebuggerResourceSetInfo( 	final IDiagramDocument p_document,
												final IEditorInput p_editorInput ) {
			super( p_document, p_editorInput );
		}

		@Override
		public void dispose() {
			stopResourceListening();
			getResourceSet().eAdapters().remove( myResourceSetListener );

			if ( terminateDebugElement == null || terminateDebugElement.isTerminated() ) {
				for ( final Iterator<Resource> it = getLoadedResourcesIterator(); it.hasNext(); ) {
					final Resource resource = it.next();
					resource.unload();
				}
			}
		}
	}
	
	@Override
	protected ElementInfo createElementInfo(Object element) 
	throws CoreException {
		if ( !( element instanceof FileEditorInput ) && !( element instanceof URIEditorInput ) ) {
			final IStatus status = new Status( 	IStatus.ERROR,
												MlsdmDiagramEditorPlugin.ID, 
												0, 
												NLS.bind( Messages.MlsdmDocumentProvider_IncorrectInputError, new Object[]{ element, FileEditorInput.class.getName(), URIEditorInput.class.getName() }),
												null );
			throw new CoreException( status );
		}
		
		final IEditorInput editorInput = (IEditorInput) element;
		final IDiagramDocument document = (IDiagramDocument) createDocument( editorInput );

		final ResourceSetInfo info = new MLSDMDebuggerResourceSetInfo(document, editorInput);	
		info.setModificationStamp( computeModificationStamp( info ) );
		info.fStatus = null;
		
		return info;
	}
	
	private long computeModificationStamp( final ResourceSetInfo p_info ) {
		int result = 0;
		
		for ( final Iterator<Resource> it = p_info.getLoadedResourcesIterator(); it.hasNext(); ) {
			final Resource nextResource = it.next();
			final IFile file = WorkspaceSynchronizer.getFile( nextResource );
			
			if ( file != null ) {
				if ( file.getLocation() != null ) {
					result += file.getLocation().toFile().lastModified();
				}
				else {
					result += file.getModificationStamp();
				}
			}
		}
		
		return result;
	}

	@Override
	protected void setDocumentContent(IDocument document, IEditorInput element)
	throws CoreException {
		assert element instanceof URIEditorInput;

		final URIEditorInput uriEditorInput = (URIEditorInput) element;

		final IDiagramDocument diagramDocument = (IDiagramDocument) document;
		
		final EditingDomain editingDomain;

		if ( uriEditorInput instanceof MLSDMDebuggerEditorInput ) {
			final MLSDMDebuggerEditorInput debugEditorInput = (MLSDMDebuggerEditorInput) uriEditorInput;
			
			if ( debugEditorInput.getEditingDomain() == null ) {
				editingDomain = createEditingDomain();
				//debugEditorInput.setEditingDomain( editingDomain );
			}
			else {
				editingDomain = debugEditorInput.getEditingDomain();
			}
		}
		else {
			editingDomain = createEditingDomain();
		}
		
		// DB: Allow this editor to be opened when Eclipse is starting (because was left open after Eclipse shut down)
		// or when a breakpoint is double clicked
//		if  ( debuggerEditorInput.getEditingDomain() == null ) {
		//	debuggerEditorInput.setEditingDomain( createEditingDomain() );
//			diagramDocument.setEditingDomain( createEditingDomain() );
//		}

		diagramDocument.setEditingDomain( (TransactionalEditingDomain) editingDomain );

		//final TransactionalEditingDomain domain = diagramDocument.getEditingDomain();

		/*
		 * Create a new story diagram if it does not exist
		 */
		if ( 	!ExtensibleURIConverterImpl.INSTANCE.exists( uriEditorInput.getURI(), null ) && 
				uriEditorInput instanceof MLSDMDebuggerEditorInput ) {
			final ResourceSet rs = new ResourceSetImpl();
			final MLSDMDebuggerEditorInput debugEditorInput = (MLSDMDebuggerEditorInput) uriEditorInput;
			final Resource diagramResource = rs.createResource( uriEditorInput.getURI() );

			final String activityUuid = debugEditorInput.getActivityUUID();
			final Activity activity = this.findActivity( activityUuid, editingDomain.getResourceSet() );

			assert activity != null;
			
			final Diagram diagram = ViewService.createDiagram(activity, ActivityEditPart.MODEL_ID,
					MlsdmDiagramEditorPlugin.DIAGRAM_PREFERENCES_HINT);

			diagramResource.getContents().add(diagram);

			try 	{
				diagramResource.save(null);
			}
			catch (IOException e) {
				throw new RuntimeException("Could not save temporary story diagram '" + uriEditorInput.getURI().toString() + "'.", e);
			}
		}

		super.setDocumentContent(diagramDocument, element);
	}

	private Activity findActivity(String activityUUID, ResourceSet resourceSet)
	{
		TreeIterator<Notifier> it = resourceSet.getAllContents();

		while (it.hasNext())
		{
			Notifier notifier = it.next();

			if (notifier instanceof Activity && ((Activity) notifier).getUuid().equals(activityUUID))
			{
				return (Activity) notifier;
			}
		}

		return null;
	}
//
//	@Override
//	protected void disposeElementInfo( 	final Object p_element,
//										final ElementInfo p_info ) {
//		assert p_element instanceof URIEditorInput;
//
//		if ( p_element instanceof MLSDMDebuggerEditorInput ) {
//			final MLSDMDebuggerEditorInput editorInput = (MLSDMDebuggerEditorInput) p_element;
//
//			if ( editorInput.getSynchronizerAdapter() != null )	{
//				try	{
//					editorInput.getSynchronizerAdapter().shutdown();
//				}
//				catch ( final RuntimeException p_ex ) {
//					throw p_ex;
//				}
//				catch ( final Exception p_ex ) {
//					throw new RuntimeException( "Could not shutdown synchronizer adapter.", p_ex );
//				}
//			}
//		}
//
//		super.disposeElementInfo( p_element, p_info );
//	}
}
